@@ -32,7 +32,8 @@ class _RequestConfig(object):
         Also, match the incoming URL if there's already a mapper, and
         store the resulting match dict in mapper_dict.
         """
-        if environ.get('HTTPS') or environ.get('wsgi.url_scheme') == 'https':
+        if environ.get('HTTPS') or environ.get('wsgi.url_scheme') == 'https' \
+           or environ.get('HTTP_X_FORWARDED_PROTO') == 'https':
             self.__shared_state.protocol = 'https'
         else:
             self.__shared_state.protocol = 'http'
@@ -49,7 +50,9 @@ class _RequestConfig(object):
                 self.__shared_state.mapper_dict = None
                 self.__shared_state.route = None
         
-        if environ.get('HTTP_HOST'):
+        if environ.get('HTTP_X_FORWARDED_HOST'):
+            self.__shared_state.host = environ['HTTP_X_FORWARDED_HOST']
+        elif environ.get('HTTP_HOST'):
             self.__shared_state.host = environ['HTTP_HOST']
         else:
             self.__shared_state.host = environ['SERVER_NAME']
@@ -122,6 +125,6 @@ def request_config(original=False):
         obj.using_request_local = False
     return _RequestConfig()
     
-from base import Mapper
-from util import url_for, redirect_to
+from routes.mapper import Mapper
+from routes.util import url_for, redirect_to
 __all__=['Mapper', 'url_for', 'redirect_to', 'request_config']
