@@ -10,13 +10,6 @@ import re
 import urllib
 from routes import request_config
 
-class literal(str):
-    """Literal class for use in systems to indicate the url is already
-    escaped"""
-    def __html__(self):
-        return self
-
-
 def _screenargs(kargs):
     """
     Private function that takes a dict, and screens it against the current 
@@ -244,7 +237,7 @@ def url_for(*args, **kargs):
         raise RouteException(
             "url_for could not generate URL. Called with args: %s %s" % \
             (args, kargs))
-    return literal(url)
+    return url
 
 def redirect_to(*args, **kargs):
     """Issues a redirect based on the arguments. 
