@@ -1,7 +1,8 @@
 """test_utils"""
-
 import os, sys, time, unittest
-from routes.util import controller_scan
+from nose.tools import eq_, assert_raises
+
+from routes.util import controller_scan, RouteException
 from routes import *
 
 class TestUtils(unittest.TestCase):
@@ -16,6 +17,20 @@ class TestUtils(unittest.TestCase):
         con.host = 'www.test.com'
         con.protocol = 'http'
         self.con = con
+    
+    def test_url_for_with_unicode(self):
+        con = self.con
+        con.mapper_dict = {}
+        
+        assert '/blog' == url_for(controller='blog')
+        assert '/blog/view/umulat' == url_for(controller='blog', action='view', id=u'umulat')
+        assert '/blog/view/umulat?other=%CE%B1%CF%83%CE%B4%CE%B3' == url_for(controller='blog', action='view', id=u'umulat',
+            other=u'\u03b1\u03c3\u03b4\u03b3')
+        
+        def raise_url():
+            return url_for(u'/some/stirng')
+        self.assertRaises(Exception, raise_url)
+        
         
     def test_url_for(self):
         con = self.con
@@ -26,6 +41,12 @@ class TestUtils(unittest.TestCase):
         self.assertEqual('https://www.test.com/viewpost', url_for(controller='post', action='view', protocol='https'))
         self.assertEqual('http://www.test.org/content', url_for(host='www.test.org'))
     
+    def test_url_raises(self):
+        con = self.con
+        con.mapper.explicit = True
+        con.mapper_dict = {}
+        assert_raises(RouteException, url_for, action='juice')
+    
     def test_url_for_with_defaults(self):
         con = self.con
         con.mapper_dict = {'controller':'blog','action':'view','id':4}
@@ -61,7 +82,7 @@ class TestUtils(unittest.TestCase):
         m.connect('category_home', 'category/:section', controller='blog', action='view', section='home')
         m.connect(':controller/:action/:id')
         m.create_regs(['content','blog','admin/comments'])
-        self.con.environ = dict(SCRIPT_NAME='', SERVER_NAME='www.example.com', PATH_INFO='/blog/view/4')
+        self.con.environ = dict(SCRIPT_NAME='', HTTP_HOST='www.example.com', PATH_INFO='/blog/view/4')
         
         self.assertEqual('/blog/view/4', url_for())
         self.assertEqual('/post/index/4', url_for(controller='post'))
@@ -69,7 +90,9 @@ class TestUtils(unittest.TestCase):
         self.assertEqual('/blog/view/2', url_for(id=2))
         self.assertEqual('/viewpost/4', url_for(controller='post', action='view', id=4))
         
-        self.con.environ = dict(SCRIPT_NAME='', SERVER_NAME='www.example.com', SERVER_PORT='8080', PATH_INFO='/blog/view/4')
+        env = dict(SCRIPT_NAME='', SERVER_NAME='www.example.com', SERVER_PORT='8080', PATH_INFO='/blog/view/4')
+        env['wsgi.url_scheme'] = 'http'
+        self.con.environ = env
         self.assertEqual('/post/index/4', url_for(controller='post'))
         self.assertEqual('http://www.example.com:8080/blog/view/4', url_for(qualified=True))
         
@@ -86,8 +109,6 @@ class TestUtils(unittest.TestCase):
         self.assertEqual('/admin/comments', url_for(controller='admin/comments'))
         self.assertEqual('/category', url_for('category_home'))
         self.assertEqual('/category/food', url_for('category_home', section='food'))
-        self.assertEqual('/category', url_for('home', action='view', section='home'))
-        self.assertEqual('/content/splash', url_for('home', controller='content'))
         self.assertEqual('/', url_for('home'))
         
     def test_with_route_names_and_defaults(self):
@@ -102,11 +123,33 @@ class TestUtils(unittest.TestCase):
         self.assertEqual('/building/wilma/port/alljacks', url_for())
         self.assertEqual('/', url_for('home'))
         
+    def test_with_route_names_and_hardcode(self):
+        m = self.con.mapper
+        self.con.mapper_dict = {}
+        m.hardcode_names = False
+        
+        m.connect('home', '', controller='blog', action='splash')
+        m.connect('category_home', 'category/:section', controller='blog', action='view', section='home')
+        m.connect('building', 'building/:campus/:building/alljacks', controller='building', action='showjacks')
+        m.connect('gallery_thumb', 'gallery/:(img_id)_thumbnail.jpg')
+        m.connect('gallery', 'gallery/:(img_id).jpg')
+        m.create_regs(['content','blog','admin/comments','building'])
 
+        self.con.mapper_dict = dict(controller='building', action='showjacks', campus='wilma', building='port')
+        self.assertEqual('/building/wilma/port/alljacks', url_for())
+        self.assertEqual('/', url_for('home'))
+        self.assertEqual('/gallery/home_thumbnail.jpg', url_for('gallery_thumb', img_id='home'))
+        self.assertEqual('/gallery/home_thumbnail.jpg', url_for('gallery', img_id='home'))
+        
+        m.hardcode_names = True
+        self.assertEqual('/gallery/home_thumbnail.jpg', url_for('gallery_thumb', img_id='home'))
+        self.assertEqual('/gallery/home.jpg', url_for('gallery', img_id='home'))
+        m.hardcode_names = False
+    
     def test_redirect_to(self):
         m = self.con.mapper
         self.con.mapper_dict = {}
-        self.con.environ = dict(SCRIPT_NAME='', SERVER_NAME='www.example.com')
+        self.con.environ = dict(SCRIPT_NAME='', HTTP_HOST='www.example.com')
         result = None
         def printer(echo):
             redirect_to.result = echo
@@ -145,17 +188,13 @@ class TestUtils(unittest.TestCase):
         self.assertEqual('/category', redirect_to.result)
         redirect_to('category_home', section='food')
         self.assertEqual('/category/food', redirect_to.result)
-        redirect_to('home', action='view', section='home')
-        self.assertEqual('/category', redirect_to.result)
-        redirect_to('home', controller='content')
-        self.assertEqual('/content/splash', redirect_to.result)
         redirect_to('home')
         self.assertEqual('/', redirect_to.result)
     
     def test_static_route(self):
         m = self.con.mapper
         self.con.mapper_dict = {}
-        self.con.environ = dict(SCRIPT_NAME='', SERVER_NAME='example.com')
+        self.con.environ = dict(SCRIPT_NAME='', HTTP_HOST='example.com')
         m.connect(':controller/:action/:id')
         m.connect('home', 'http://www.groovie.org/', _static=True)
         m.connect('space', '/nasa/images', _static=True)
@@ -169,7 +208,7 @@ class TestUtils(unittest.TestCase):
     def test_static_route_with_script(self):
         m = self.con.mapper
         self.con.mapper_dict = {}
-        self.con.environ = dict(SCRIPT_NAME='/webapp', SERVER_NAME='example.com')
+        self.con.environ = dict(SCRIPT_NAME='/webapp', HTTP_HOST='example.com')
         m.connect(':controller/:action/:id')
         m.connect('home', 'http://www.groovie.org/', _static=True)
         m.connect('space', '/nasa/images', _static=True)
@@ -184,7 +223,7 @@ class TestUtils(unittest.TestCase):
     def test_no_named_path(self):
         m = self.con.mapper
         self.con.mapper_dict = {}
-        self.con.environ = dict(SCRIPT_NAME='', SERVER_NAME='example.com')
+        self.con.environ = dict(SCRIPT_NAME='', HTTP_HOST='example.com')
         m.connect(':controller/:action/:id')
         m.connect('home', 'http://www.groovie.org/', _static=True)
         m.connect('space', '/nasa/images', _static=True)
@@ -199,7 +238,7 @@ class TestUtils(unittest.TestCase):
         m = self.con.mapper
         self.con.mapper_dict = {}
         m.append_slash = True
-        self.con.environ = dict(SCRIPT_NAME='', SERVER_NAME='example.com')
+        self.con.environ = dict(SCRIPT_NAME='', HTTP_HOST='example.com')
         m.connect(':controller/:action/:id')
         m.connect('home', 'http://www.groovie.org/', _static=True)
         m.connect('space', '/nasa/images', _static=True)
@@ -215,7 +254,7 @@ class TestUtils(unittest.TestCase):
     def test_no_named_path_with_script(self):
         m = self.con.mapper
         self.con.mapper_dict = {}
-        self.con.environ = dict(SCRIPT_NAME='/webapp', SERVER_NAME='example.com')
+        self.con.environ = dict(SCRIPT_NAME='/webapp', HTTP_HOST='example.com')
         m.connect(':controller/:action/:id')
         m.connect('home', 'http://www.groovie.org/', _static=True)
         m.connect('space', '/nasa/images', _static=True)
@@ -240,7 +279,7 @@ class TestUtils(unittest.TestCase):
             return kargs
         
         self.con.mapper_dict = {}
-        self.con.environ = dict(SCRIPT_NAME='', SERVER_NAME='example.com')
+        self.con.environ = dict(SCRIPT_NAME='', HTTP_HOST='example.com')
 
         m = Mapper()
         m.connect(':controller/:(action)-:(id).html')
@@ -249,8 +288,8 @@ class TestUtils(unittest.TestCase):
         m.create_regs(['content','archives','admin/comments'])
         self.con.mapper = m
         
-        self.assertEqual(None, url_for(controller='content', action='view'))
-        self.assertEqual(None, url_for(controller='content'))
+        self.assertRaises(Exception, url_for, controller='content', action='view')
+        self.assertRaises(Exception, url_for, controller='content')
         
         self.assertEqual('/content/view-3.html', url_for(controller='content', action='view', id=3))
         self.assertEqual('/content/index-2.html', url_for(controller='content', id=2))
@@ -294,6 +333,7 @@ class TestUtils(unittest.TestCase):
     def test_with_http_environ(self):
         base_environ = dict(SCRIPT_NAME='', SERVER_PORT='1080', PATH_INFO='/', 
             HTTP_HOST='example.com', SERVER_NAME='example.com')
+        base_environ['wsgi.url_scheme'] = 'http'
         self.con.environ = base_environ.copy()
         self.con.mapper_dict = {}
 
@@ -427,7 +467,176 @@ class TestUtils(unittest.TestCase):
         assert {'action':'index', 'controller':'content','id':None} == m.match('/content')
         assert {'action':'index', 'controller':'users','id':None} == m.match('/users')
         assert {'action':'index', 'controller':'admin/users','id':None} == m.match('/admin/users')        
+
+class TestUtilsWithExplicit(unittest.TestCase):
+    def setUp(self):
+        m = Mapper(explicit=True)
+        m.connect('archive/:year/:month/:day', controller='blog', action='view', month=None, day=None,
+                  requirements={'month':'\d{1,2}','day':'\d{1,2}'})
+        m.connect('viewpost/:id', controller='post', action='view', id=None)
+        m.connect(':controller/:action/:id')
+        con = request_config()
+        con.mapper = m
+        con.host = 'www.test.com'
+        con.protocol = 'http'
+        self.con = con
+        
+    def test_url_for(self):
+        con = self.con
+        con.mapper_dict = {}
+        
+        self.assertRaises(Exception, url_for, controller='blog')
+        self.assertRaises(Exception, url_for)
+        self.assertEqual('/blog/view/3', url_for(controller='blog', action='view', id=3))
+        self.assertEqual('https://www.test.com/viewpost', url_for(controller='post', action='view', protocol='https'))
+        self.assertEqual('http://www.test.org/content/view/2', url_for(host='www.test.org', controller='content', action='view', id=2))
     
+    def test_url_for_with_defaults(self):
+        con = self.con
+        con.mapper_dict = {'controller':'blog','action':'view','id':4}
+        
+        self.assertRaises(Exception, url_for)
+        self.assertRaises(Exception, url_for, controller='post')
+        self.assertRaises(Exception, url_for, id=2)
+        self.assertEqual('/viewpost/4', url_for(controller='post', action='view', id=4))
+        
+        con.mapper_dict = {'controller':'blog','action':'view','year':2004}
+        self.assertRaises(Exception, url_for, month=10)
+        self.assertRaises(Exception, url_for, month=9, day=2)
+        self.assertRaises(Exception, url_for, controller='blog', year=None)
+    
+    def test_url_for_with_more_defaults(self):
+        con = self.con
+        con.mapper_dict = {'controller':'blog','action':'view','id':4}
+        
+        self.assertRaises(Exception, url_for)
+        self.assertRaises(Exception, url_for, controller='post')
+        self.assertRaises(Exception, url_for, id=2)
+        self.assertEqual('/viewpost/4', url_for(controller='post', action='view', id=4))
+        
+        con.mapper_dict = {'controller':'blog','action':'view','year':2004}
+        self.assertRaises(Exception, url_for, month=10)
+        self.assertRaises(Exception, url_for)
+    
+    def test_url_for_with_defaults_and_qualified(self):
+        m = self.con.mapper
+        m.connect('home', '', controller='blog', action='splash')
+        m.connect('category_home', 'category/:section', controller='blog', action='view', section='home')
+        m.connect(':controller/:action/:id')
+        m.create_regs(['content','blog','admin/comments'])
+        env = dict(SCRIPT_NAME='', SERVER_NAME='www.example.com', SERVER_PORT='80', PATH_INFO='/blog/view/4')
+        env['wsgi.url_scheme'] = 'http'
+        
+        self.con.environ = env
+        
+        self.assertRaises(Exception, url_for)
+        self.assertRaises(Exception, url_for, controller='post')
+        self.assertRaises(Exception, url_for, id=2)
+        self.assertRaises(Exception, url_for, qualified=True, controller='blog', id=4)
+        self.assertEqual('http://www.example.com/blog/view/4', url_for(qualified=True, controller='blog', action='view', id=4))
+        self.assertEqual('/viewpost/4', url_for(controller='post', action='view', id=4))
+        
+        env = dict(SCRIPT_NAME='', SERVER_NAME='www.example.com', SERVER_PORT='8080', PATH_INFO='/blog/view/4')
+        env['wsgi.url_scheme'] = 'http'
+        self.con.environ = env
+        self.assertRaises(Exception, url_for, controller='post')
+        self.assertEqual('http://www.example.com:8080/blog/view/4', url_for(qualified=True, controller='blog', action='view', id=4))
+        
+    
+    def test_with_route_names(self):
+        m = self.con.mapper
+        self.con.mapper_dict = {}
+        m.connect('home', '', controller='blog', action='splash')
+        m.connect('category_home', 'category/:section', controller='blog', action='view', section='home')
+        m.create_regs(['content','blog','admin/comments'])
+
+        self.assertRaises(Exception, url_for, controller='content', action='view')
+        self.assertRaises(Exception, url_for, controller='content')
+        self.assertRaises(Exception, url_for, controller='admin/comments')
+        self.assertEqual('/category', url_for('category_home'))
+        self.assertEqual('/category/food', url_for('category_home', section='food'))
+        self.assertRaises(Exception, url_for, 'home', controller='content')
+        self.assertEqual('/', url_for('home'))
+
+    def test_with_route_names_and_nomin(self):
+        m = self.con.mapper
+        m.minimization = False
+        self.con.mapper_dict = {}
+        m.connect('home', '', controller='blog', action='splash')
+        m.connect('category_home', 'category/:section', controller='blog', action='view', section='home')
+        m.create_regs(['content','blog','admin/comments'])
+
+        self.assertRaises(Exception, url_for, controller='content', action='view')
+        self.assertRaises(Exception, url_for, controller='content')
+        self.assertRaises(Exception, url_for, controller='admin/comments')
+        self.assertEqual('/category/home', url_for('category_home'))
+        self.assertEqual('/category/food', url_for('category_home', section='food'))
+        self.assertRaises(Exception, url_for, 'home', controller='content')
+        self.assertEqual('/', url_for('home'))
+
+    def test_with_route_names_and_defaults(self):
+        m = self.con.mapper
+        self.con.mapper_dict = {}
+        m.connect('home', '', controller='blog', action='splash')
+        m.connect('category_home', 'category/:section', controller='blog', action='view', section='home')
+        m.connect('building', 'building/:campus/:building/alljacks', controller='building', action='showjacks')
+        m.create_regs(['content','blog','admin/comments','building'])
+
+        self.con.mapper_dict = dict(controller='building', action='showjacks', campus='wilma', building='port')
+        self.assertRaises(Exception, url_for)
+        self.assertEqual('/building/wilma/port/alljacks', url_for(controller='building', action='showjacks', campus='wilma', building='port'))
+        self.assertEqual('/', url_for('home'))
+
+    def test_with_resource_route_names(self):
+        m = Mapper()
+        self.con.mapper = m
+        self.con.mapper_dict = {}
+        m.resource('message', 'messages', member={'mark':'GET'}, collection={'rss':'GET'})
+        m.create_regs(['messages'])
+
+        self.assertRaises(Exception, url_for, controller='content', action='view')
+        self.assertRaises(Exception, url_for, controller='content')
+        self.assertRaises(Exception, url_for, controller='admin/comments')
+        self.assertEqual('/messages', url_for('messages'))
+        self.assertEqual('/messages/rss', url_for('rss_messages'))
+        self.assertEqual('/messages/4', url_for('message', id=4))
+        self.assertEqual('/messages/4/edit', url_for('edit_message', id=4))
+        self.assertEqual('/messages/4/mark', url_for('mark_message', id=4))
+        self.assertEqual('/messages/new', url_for('new_message'))
+        
+        self.assertEqual('/messages.xml', url_for('formatted_messages', format='xml'))
+        self.assertEqual('/messages/rss.xml', url_for('formatted_rss_messages', format='xml'))
+        self.assertEqual('/messages/4.xml', url_for('formatted_message', id=4, format='xml'))
+        self.assertEqual('/messages/4/edit.xml', url_for('formatted_edit_message', id=4, format='xml'))
+        self.assertEqual('/messages/4/mark.xml', url_for('formatted_mark_message', id=4, format='xml'))
+        self.assertEqual('/messages/new.xml', url_for('formatted_new_message', format='xml'))
+
+    def test_with_resource_route_names_and_nomin(self):
+        m = Mapper()
+        self.con.mapper = m
+        self.con.mapper_dict = {}
+        m.minimization = False
+        m.resource('message', 'messages', member={'mark':'GET'}, collection={'rss':'GET'})
+        m.create_regs(['messages'])
+
+        self.assertRaises(Exception, url_for, controller='content', action='view')
+        self.assertRaises(Exception, url_for, controller='content')
+        self.assertRaises(Exception, url_for, controller='admin/comments')
+        self.assertEqual('/messages', url_for('messages'))
+        self.assertEqual('/messages/rss', url_for('rss_messages'))
+        self.assertEqual('/messages/4', url_for('message', id=4))
+        self.assertEqual('/messages/4/edit', url_for('edit_message', id=4))
+        self.assertEqual('/messages/4/mark', url_for('mark_message', id=4))
+        self.assertEqual('/messages/new', url_for('new_message'))
+        
+        self.assertEqual('/messages.xml', url_for('formatted_messages', format='xml'))
+        self.assertEqual('/messages/rss.xml', url_for('formatted_rss_messages', format='xml'))
+        self.assertEqual('/messages/4.xml', url_for('formatted_message', id=4, format='xml'))
+        self.assertEqual('/messages/4/edit.xml', url_for('formatted_edit_message', id=4, format='xml'))
+        self.assertEqual('/messages/4/mark.xml', url_for('formatted_mark_message', id=4, format='xml'))
+        self.assertEqual('/messages/new.xml', url_for('formatted_new_message', format='xml'))
+        
+
 if __name__ == '__main__':
     unittest.main()
 else:
